@@ -1,4 +1,4 @@
-.getFilesNames <- function(path, sep=';') {
+.getFilesNames <- function(path, sep=";") {
 ################################################################################
 # DESCRIPTION:
 # As a function of a path and a file extension, gets all the file names there.
@@ -7,8 +7,8 @@
   files <- list.files(path=path, full.names=T)
   
   # Match pattern
-  if (sep %in% c('xls','xlsx')) {
-    pat <- '[.xls][a-z]{1}$'
+  if (sep %in% c("xls","xlsx")) {
+    pat <- "[.xls][a-z]{1}$"
   }
   else {
     if (sep %in% c(";", ",")) sep <- "csv"
@@ -25,7 +25,7 @@
     stop(nfiles, ' files found.\nCannot\' continue') 
   }
   else {
-    cat(nfiles, 'files found...\n')
+    message(nfiles, ' files found...')
   }
   return(files)
 }
@@ -51,8 +51,12 @@ genMoreInfo <- function(path, encoding=getOption("encoding"), sep=";",
     
   # Extracts the unique list of variables
   x <- unique(
-    subset(x,subset=type != 'date' & is.dim.tab==F, 
-           select=c(id, label))
+    # subset(x,subset=type != 'date' & is.dim.tab==F, 
+           # select=c(id, label))
+    x[
+      with(x, type!='date' & !is.dim.tab),
+      colnames(x) %in% c('id','label'),
+      FALSE]
     )
   
   x <- data.frame(cbind(x, description=NA, topic=NA, url=NA, totalName=NA, 
@@ -78,10 +82,10 @@ genMoreInfo <- function(path, encoding=getOption("encoding"), sep=";",
     }
     ERR<-try(write.table(x, file=output, quote=F, na="NA", sep='\t'))
     if (class(ERR)!='try-error') {
-      cat('DSPL Configfile written correctly at',output,'\n')
+      message("DSPL Configfile written correctly at\n",normalizePath(output))
     }
     else {
-      cat('An error has occurred during the file writing at',output,'\n')
+      stop("An error has occurred during the file writing at:\n",normalizePath(output))
     }
   }
   
@@ -89,10 +93,10 @@ genMoreInfo <- function(path, encoding=getOption("encoding"), sep=";",
   else if (action %in% c('replace','merge')) {
     ERR<-try(write.table(x, file=output, quote=F, na="NA", sep='\t'))
     if (class(ERR)!='try-error') {
-      cat('DSPL Configfile written correctly at',output,'\n')
+      message("DSPL Configfile written correctly at:\n",normalizePath(output))
     }
     else {
-      cat('An error has occurred during the file writing at',output,'\n')
+      stop("An error has occurred during the file writing at:\n",normalizePath(output))
     }
   }
 }
@@ -104,10 +108,10 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
 # taframe. Also determinates which field is dim or metric.
 ################################################################################  
   # Checking if xls option is Ok
-  if (sep == 'xls') load.xlsx <- require(XLConnect) else load.xlsx <- TRUE
+  #if (sep == 'xls') load.xlsx <- require(XLConnect) else load.xlsx <- TRUE
   
-  if (!load.xlsx) stop("In order to read MS Excel files ",
-                       "you need to get the package \'XLConnect\' first.")  
+  #if (!load.xlsx) stop("In order to read MS Excel files ",
+  #                     "you need to get the package \'XLConnect\' first.")  
   
   # Timeframe metrics
   metrics <- matrix(c(
@@ -182,13 +186,13 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
                x=data, file=con, na='', sep=',',quote=F,row.names=F,dec='.'
                )
 
-             cat(
-               gsub(".*(/|\\\\)","",x),'analized correctly, ordered by ', ord,
-               ' and exported as csv\n'
+             message(
+               gsub(".*(/|\\\\)","",x)," analized correctly, ordered by ", ord,
+               " and exported as csv"
                )
            }
            else {
-             cat(x,'analized correctly\n')
+             message(x,"analized correctly")
            }
            return(var)
          }
@@ -213,7 +217,7 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
   
   # Setting which concepts extends geo:location
   
-  geo <-subset(vars, is.dim.tab == T, select=c(id, slice))
+  geo <-vars[with(vars, is.dim.tab == T), colnames(vars) %in% c("id", "slice"), FALSE]
   geo <- by(
     geo, geo$slice, 
     function(x) all(c('longitude', 'latitude') %in% unlist(x['id']))
@@ -226,10 +230,15 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
   return(vars)
 }
 
-.getMoreInfo <- function(source,target, encoding="unknown") {
+.getMoreInfo <- function(source,target, encoding="unknown", id='id', label='label') {
 ################################################################################
 # Reads from a tab file generated by genMoreInfo as a complement info to concepts
 ################################################################################  
+  
+  # Partial fix (should work, need to see
+  # http://stackoverflow.com/questions/23475309/in-r-is-it-possible-to-suppress-note-no-visible-binding-for-global-variable)
+  # type <- is.dim.tab <- id <- label <- freq <- NULL
+  
   if (!is.na(source)) {
     
     options(stringsAsFactors=F)
@@ -256,9 +265,9 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
     }
     
     # Merges source and target
-    source <- subset(source, id %in% target$id)
-    target$label <- source$label
-    source <- subset(source, select=-label)
+    source <- source[which(source[[id]] %in% target$id),] # subset(source, id %in% target$id)
+    target$label <- source[[label]] # source$label
+    source <- source[,colnames(source)!=label,FALSE] #subset(source, select=-label)
     target <- merge(target, source, by=c('id'))
   }
   return(target)
@@ -335,104 +344,108 @@ seekTables <- function(files, encoding, sep, output = NA, replace = T, dec) {
   colnames(val)[3] <- 'ref'
   colnames(val)[7] <- 'geo'
   
-  if (NCOL(val) > 1) {
-    fun <- function(x, ...) {apply(x, MARGIN = 1,...)}
-  } else {
-    fun <- function(x, FUN) {FUN(x)}
-  }
-  
-  fun(val, FUN= 
-    function(x) {
-      x['is.dim.tab'] <- gsub(' ','',x['is.dim.tab'])
-      x['geo'] <- gsub(' ','',x['geo'])
-      
-      if (x["ref"] == 'string') {ATT <- c(x['id'], extends='entity:entity')}
-      if (x['ref'] == 'string' && x['geo']=='TRUE') {ATT <- c(x['id'], extends='geo:location')}
-      if (x['ref'] != 'string') {ATT <- c(x['id'])}
-      
-      # in the case of not being a dimensional concept
-      if (x["concept.type"]!="dimension") {
-        tempnode0 <- newXMLNode('concept', attrs=ATT, parent=parent)
-        
-        # Starts Adding info
-        tempnode1 <- newXMLNode('info', parent=tempnode0)
-                     
-        tempnode2 <- newXMLNode('name', parent=tempnode1)
-        
-        # Adds a description
-        if (!is.na(x['description'])) {
-          description <- as.character(x['description'])
-          tempnode3 <- newXMLNode('description', parent=tempnode1)
-          newXMLNode('value', parent=tempnode3, attrs=c('xml:lang'=lang[1]), description)
-        }
-        
-        # URL node
-        if (!is.na(x['url'])) {
-          url <- as.character(x['url'])
-          tempnode4 <- newXMLNode('url', parent=tempnode1)
-          newXMLNode('value', parent=tempnode4, attrs=c('xml:lang'=lang[1]), url)
-        }
-        
-        # Here should start the multilanguage loop
-        newXMLNode('value', parent=tempnode2, attrs=c('xml:lang'=lang[1]),
-                   suppressNamespaceWarning=T, x['label'])
-        
-        # Adds a topic category
-        if (!is.na(x['topicid'])) {
-          topicref <- as.character(x['topicid'])
-          newXMLNode('topic', parent=tempnode0, attrs=c(ref=topicref))
-        }
-        
-        # Adds the data type specification
-        newXMLNode('type', attrs=c(x['ref']), parent=tempnode0)
-        
-      # in the case of being a dimensional concept
-      } else {
-        tempnode0 <- newXMLNode('concept', attrs=ATT, parent=parent)
-        
-        # Starts adding info
-        tempnode1 <- newXMLNode('info', parent=tempnode0)
-        
-        # Name Node
-        tempnode2 <- newXMLNode('name', parent=tempnode1)
-        newXMLNode('value', parent=tempnode2, attrs=c('xml:lang'=lang[1]),
-                   suppressNamespaceWarning=T, x['label'])
-        
-        # Description node
-        if (!is.na(x['description'])) {
-          description <- as.character(x['description'])
-          tempnode3 <- newXMLNode('description', parent=tempnode1)
-          newXMLNode('value', parent=tempnode3, attrs=c('xml:lang'=lang[1]), description)
-        }
-        
-        # URL node
-        if (!is.na(x['url'])) {
-          url <- as.character(x['url'])
-          tempnode4 <- newXMLNode('url', parent=tempnode1)
-          newXMLNode('value', parent=tempnode4, attrs=c('xml:lang'=lang[1]), url)
-        }
-        
-        # Plural name node
-        if (!is.na(x['totalName'])) {
-          pluralName <- as.character(x['pluralName'])
-          tempnode6 <- newXMLNode('pluralName', parent=tempnode1)
-          newXMLNode('value', parent=tempnode6, attrs=c('xml:lang'=lang[1]), pluralName)
-        }        
-        
-        # Total name node
-        if (!is.na(x['totalName'])) {
-          totalName <- as.character(x['totalName'])
-          tempnode5 <- newXMLNode('totalName', parent=tempnode1)
-          newXMLNode('value', parent=tempnode5, attrs=c('xml:lang'=lang[1]),totalName)
-        }
+#   if (NCOL(val) > 1) {
+#     fun <- function(x, ...) {apply(x, MARGIN = 1,...)}
+#   } else {
+#     fun <- function(x, FUN) {FUN(x)}
+#   }
+#   
+#   fun(val, .addConcepts_sub)
+  if (ncol(val)>1) apply(val,.addConcepts_sub,MARGIN = 1,parent=parent,lang=lang)
+  else .addConcepts(val,parent=parent,lang=lang)
+}
 
-        newXMLNode('type', parent=tempnode0, attrs=c(x['ref']))
-        newXMLNode('table', parent=tempnode0, attrs=
-          c(ref=paste(x['dim.tab.ref'],'_table',sep='')))
-                   
+.addConcepts_sub <- function(x,parent,lang) {
+  function(x) {
+    x['is.dim.tab'] <- gsub(' ','',x['is.dim.tab'])
+    x['geo'] <- gsub(' ','',x['geo'])
+    
+    if (x["ref"] == 'string') {ATT <- c(x['id'], extends='entity:entity')}
+    if (x['ref'] == 'string' && x['geo']=='TRUE') {ATT <- c(x['id'], extends='geo:location')}
+    if (x['ref'] != 'string') {ATT <- c(x['id'])}
+    
+    # in the case of not being a dimensional concept
+    if (x["concept.type"]!="dimension") {
+      tempnode0 <- newXMLNode('concept', attrs=ATT, parent=parent)
+      
+      # Starts Adding info
+      tempnode1 <- newXMLNode('info', parent=tempnode0)
+      
+      tempnode2 <- newXMLNode('name', parent=tempnode1)
+      
+      # Adds a description
+      if (!is.na(x['description'])) {
+        description <- as.character(x['description'])
+        tempnode3 <- newXMLNode('description', parent=tempnode1)
+        newXMLNode('value', parent=tempnode3, attrs=c('xml:lang'=lang[1]), description)
       }
+      
+      # URL node
+      if (!is.na(x['url'])) {
+        url <- as.character(x['url'])
+        tempnode4 <- newXMLNode('url', parent=tempnode1)
+        newXMLNode('value', parent=tempnode4, attrs=c('xml:lang'=lang[1]), url)
+      }
+      
+      # Here should start the multilanguage loop
+      newXMLNode('value', parent=tempnode2, attrs=c('xml:lang'=lang[1]),
+                 suppressNamespaceWarning=T, x['label'])
+      
+      # Adds a topic category
+      if (!is.na(x['topicid'])) {
+        topicref <- as.character(x['topicid'])
+        newXMLNode('topic', parent=tempnode0, attrs=c(ref=topicref))
+      }
+      
+      # Adds the data type specification
+      newXMLNode('type', attrs=c(x['ref']), parent=tempnode0)
+      
+      # in the case of being a dimensional concept
+    } else {
+      tempnode0 <- newXMLNode('concept', attrs=ATT, parent=parent)
+      
+      # Starts adding info
+      tempnode1 <- newXMLNode('info', parent=tempnode0)
+      
+      # Name Node
+      tempnode2 <- newXMLNode('name', parent=tempnode1)
+      newXMLNode('value', parent=tempnode2, attrs=c('xml:lang'=lang[1]),
+                 suppressNamespaceWarning=T, x['label'])
+      
+      # Description node
+      if (!is.na(x['description'])) {
+        description <- as.character(x['description'])
+        tempnode3 <- newXMLNode('description', parent=tempnode1)
+        newXMLNode('value', parent=tempnode3, attrs=c('xml:lang'=lang[1]), description)
+      }
+      
+      # URL node
+      if (!is.na(x['url'])) {
+        url <- as.character(x['url'])
+        tempnode4 <- newXMLNode('url', parent=tempnode1)
+        newXMLNode('value', parent=tempnode4, attrs=c('xml:lang'=lang[1]), url)
+      }
+      
+      # Plural name node
+      if (!is.na(x['totalName'])) {
+        pluralName <- as.character(x['pluralName'])
+        tempnode6 <- newXMLNode('pluralName', parent=tempnode1)
+        newXMLNode('value', parent=tempnode6, attrs=c('xml:lang'=lang[1]), pluralName)
+      }        
+      
+      # Total name node
+      if (!is.na(x['totalName'])) {
+        totalName <- as.character(x['totalName'])
+        tempnode5 <- newXMLNode('totalName', parent=tempnode1)
+        newXMLNode('value', parent=tempnode5, attrs=c('xml:lang'=lang[1]),totalName)
+      }
+      
+      newXMLNode('type', parent=tempnode0, attrs=c(x['ref']))
+      newXMLNode('table', parent=tempnode0, attrs=
+                   c(ref=paste(x['dim.tab.ref'],'_table',sep='')))
+      
     }
-  )
+  }
 }
 
 .addSlices <- function(tableid, sliceatt, parent) {
@@ -547,16 +560,24 @@ dspl <- function(
   vars <- seekTables(files=files, encoding=encoding, sep=sep, 
                      output=temp.path, replace=replace, dec=dec)
   
-  dims <- subset(vars, concept.type=='dimension', select=c(id, slice, concept.type))
+  dims <- vars[
+    with(vars, concept.type=='dimension'),
+    colnames(vars) %in% c('id', 'slice', 'concept.type'), FALSE
+    ]
+  #subset(vars, concept.type=='dimension', select=c(id, slice, concept.type)) 
   
   # Identifying if there is any duplicated slice
-  .checkSlices(dims=dims$id, by=dims$slice)
+  with(dims, .checkSlices(dims=id, by=slice))
   
   vars <- .checkDuplConcepts(concepts=vars)
   
   # Creates a unique concept list
   varConcepts <- unique(
-    subset(vars,subset=type != 'date' & is.dim.tab==F, select=-slice)
+    # subset(vars,subset=type != 'date' & is.dim.tab==F, select=-slice) 
+    vars[
+      with(vars, type!='date' & is.dim.tab==FALSE),
+      colnames(vars)!='slice',FALSE
+      ]
     )
   
   # Checks if there is a moreinfo file
@@ -568,7 +589,8 @@ dspl <- function(
     targetNamespace=targetNamespace), namespace=c(xsi=
       'http://www.w3.org/2001/XMLSchema-instance'))
   
-  try(xmlAttrs(dspl)['xmlns'] <- 'http://schemas.google.com/dspl/2010', silent=T)
+  tmpattrs <- xmlAttrs(dspl)
+  try(tmpattrs['xmlns'] <- 'http://schemas.google.com/dspl/2010', silent=T)
   
   # Definiciones dspl
   
@@ -626,9 +648,9 @@ dspl <- function(
   newXMLCommentNode('Slices Definitions', parent=dspl)
   slices <- newXMLNode('slices', parent = dspl)
   .addSlices(
-    tableid=subset(vars, is.dim.tab != T, slice),
-    sliceatt=subset(vars, is.dim.tab != T),
-    parent=slices
+    tableid  = vars[with(vars, !is.dim.tab),, FALSE][['slice']],
+    sliceatt = vars[with(vars, !is.dim.tab),, FALSE], # subset(vars, is.dim.tab != T),
+    parent   = slices
     )
   
   # TABLES
@@ -637,16 +659,20 @@ dspl <- function(
   .addTables(tableid=vars$slice,tableatt=vars,parent=tables, format=timeFormat)  
   
   # Building ouput
-  .dimtabs <- unique(subset(vars, subset=is.dim.tab, select=slice))
-  .slices <- unique(subset(vars, select=slice))
-  .concepts <- unique(subset(vars, select=label))
-  .dims <- unique(
-    subset(
-      vars, 
-      subset=is.dim.tab & !(label %in% c('name', 'latitude','longitude','colour')),
-      select=label
-      )
-    )
+  .dimtabs  <- unique(vars[with(vars, is.dim.tab),,FALSE][['slice']]) #subset(vars, subset=is.dim.tab, select=slice))
+  .slices   <- unique(vars[['slice']]) #subset(vars, select=slice))
+  .concepts <- unique(vars[['label']]) #subset(vars, select=label))
+  .dims <- unique(  
+#     subset(
+#       vars, 
+#       subset=is.dim.tab & !(label %in% c('name', 'latitude','longitude','colour')),
+#       select=label
+#       )
+    vars[ 
+      with(vars, is.dim.tab & !(label %in% c('name', 'latitude','longitude','colour'))),
+      colnames(vars)=='label',
+      FALSE
+    ]) 
   
   lapply(c(.dimtabs, .slices, .concepts, .dims), function(x) names(x) <- 'Name')
   
@@ -665,7 +691,7 @@ dspl <- function(
         .dims, pde.statistics
     ), .Names=c('dspl', 'concepts.by.table', 'dimtabs', 'slices', 'concepts',
                   'dimentions','statistics'))
-  class(result) <- c('dspl')
+  class(result) <- c("dspl")
   
   # If an output file is specified, it writes it on it
   if (is.na(output)) {
@@ -673,12 +699,12 @@ dspl <- function(
   
   } else {
     path <- paste(temp.path,'/metadata.xml',sep='')
-    print.dspl(x=result, path=path, replace=replace)
+    print.dspl(x=result, path=path, replace=replace, quiet=T)
     
     # Zipping the files
     tozip <- list.files(temp.path, full.names=T, pattern="csv$|xml$")
     zip(output ,tozip,flags='-r9jm')
     
-    return(paste('Metadata created successfully at:', output))
+    message("Metadata created successfully at:\n", normalizePath(output))
   }
-}
\ No newline at end of file
+}
